package com.winhc.kafka.consumer;

import com.winhc.service.RelationService;
import com.winhc.utils.CompanyUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.context.annotation.Bean;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.annotation.TopicPartition;
import org.springframework.kafka.listener.ConsumerAwareListenerErrorHandler;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author π
 * @Description:
 * @date 2021/1/8 16:04
 */
@Slf4j
//@Service
@AllArgsConstructor
public class KafkaConsumerNeo4j {

    private final Map<String, RelationService> map;

    @KafkaListener(id = "${spring.kafka.topic_company_node}"
//            topicPartitions = {@TopicPartition(topic = "${spring.kafka.topic_company_node}")
//                    //,partitions = {"0"})
//                    //partitionOffsets = @PartitionOffset(partition = "0", initialOffset = "0"))
//            },
            , topics = "${spring.kafka.topic_company_node}"
            , groupId = "${spring.kafka.consumer.group-id}", containerFactory = "containerFactory", errorHandler = "consumerAwareListenerErrorHandler")
    public void consumerCompanyNode(List<ConsumerRecord<?, ?>> records) {
        //保存
        map.get("companyNodeServiceImpl").save(CompanyUtils.map(records));
        //提交偏移量
        //ack.acknowledge();
    }

    @KafkaListener(id = "${spring.kafka.topic_holder_v1}"
            , topics = "${spring.kafka.topic_holder_v1}"
            , groupId = "${spring.kafka.consumer.group-id}", containerFactory = "containerFactory", errorHandler = "consumerAwareListenerErrorHandler")
    public void consumerHolderV1(List<ConsumerRecord<?, ?>> records//, Acknowledgment ack
    ) {
        //保存
        map.get("holderRelationV1ServiceImpl").save(CompanyUtils.map(records));
    }

    @KafkaListener(id = "${spring.kafka.topic_holder_v2}"
            , topics = "${spring.kafka.topic_holder_v2}"
            , groupId = "${spring.kafka.consumer.group-id}", containerFactory = "containerFactory", errorHandler = "consumerAwareListenerErrorHandler")
    public void consumerHolderV2(List<ConsumerRecord<?, ?>> records) {
        //保存
        map.get("holderRelationV2ServiceImpl").save(CompanyUtils.map(records));
    }

    @KafkaListener(id = "${spring.kafka.topic_staff_relation}"
            , topics = "${spring.kafka.topic_staff_relation}"
            , groupId = "${spring.kafka.consumer.group-id}", containerFactory = "containerFactory", errorHandler = "consumerAwareListenerErrorHandler")
    public void consumerStaffRelationV1(List<ConsumerRecord<?, ?>> records) {
        //保存
        map.get("staffRelationServiceImpl").save(CompanyUtils.map(records));
    }

    @KafkaListener(id = "${spring.kafka.topic_legal_entity_relation_v1}"
            , topics = "${spring.kafka.topic_legal_entity_relation_v1}"
            , groupId = "${spring.kafka.consumer.group-id}", containerFactory = "containerFactory", errorHandler = "consumerAwareListenerErrorHandler")
    public void consumerLegalEntityRelationV1(List<ConsumerRecord<?, ?>> records) {
        //保存
        map.get("legalEntityRelationV1ServiceImpl").save(CompanyUtils.map(records));
    }

    @KafkaListener(id = "${spring.kafka.topic_legal_entity_relation_v2}"
            , topics = "${spring.kafka.topic_legal_entity_relation_v2}"
            , groupId = "${spring.kafka.consumer.group-id}", containerFactory = "containerFactory", errorHandler = "consumerAwareListenerErrorHandler")
    public void consumerLegalEntityRelationV2(List<ConsumerRecord<?, ?>> records) {
        //保存
        map.get("legalEntityRelationV2ServiceImpl").save(CompanyUtils.map(records));
    }


    /**
     * 因为手动确认消费，若消费失败，记录重刷
     */
    @Bean("consumerAwareListenerErrorHandler")
    public ConsumerAwareListenerErrorHandler dealError() {
        return (message, e, consumer) -> {
            List<ConsumerRecord> records = (List<ConsumerRecord>) message.getPayload();
            log.error("consumer error: {}",e.getMessage());
            //System.out.println("consumer error:" + e.getMessage());
            //System.out.println("consumer 1:" + consumer.toString());
            //System.out.println("consumer 2:" + records);
            log.error("consumer message: {}",records.get(0).toString());
            // TODO 将失败的记录保存到数据库，再用定时任务查询记录，并重刷数据
            return null;
        };
    }

}
