package com.winhc.service.impl;

import com.winhc.common.enums.CompanyEnum;
import com.winhc.service.RelationService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.neo4j.driver.Driver;
import org.neo4j.driver.Result;
import org.neo4j.driver.Session;
import org.springframework.stereotype.Service;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author π
 * @Description:主要成员 公司->公司
 * @date 2021/1/11 10:03
 */
@Slf4j
@Service("legalEntityRelationV2ServiceImpl")
@AllArgsConstructor
public class LegalEntityRelationV2ServiceImpl implements RelationService {

    private final Driver driver;

    @Override
    public String save(List<Map<String, Object>> batch_list) {
        if (batch_list.isEmpty()) return null;
        long start = System.currentTimeMillis();

        Session session = driver.session();
        final String cql = "WITH  {batch_list} AS batch_list \n" +
                "UNWIND batch_list AS row \n" +
                "MERGE(s:" + CompanyEnum.Lable.COMPANY.code + "{company_id:row.start_id}) \n" +
                "SET s.name=row.start_name, s.company_id=row.start_id \n" +
                "MERGE(e:" + CompanyEnum.Lable.COMPANY.code + "{company_id:row.end_id}) \n" +
                "SET e.name=row.end_name, e.company_id=row.end_id \n" +
                "WITH s,e,row \n" +
                "MERGE(s)-[r:" + CompanyEnum.Lable.法人.code + "]->(e) \n" +
                "SET r.deleted=row.deleted \n";
        Map parameters = new HashMap() {{
            put("batch_list", batch_list);
        }};
        //log.info("cql:" + cql);
        String data = session.writeTransaction(tx -> {
            Result result = tx.run(cql, parameters);
            return "success";
        });
        session.close();
        log.info("class:{} | save size:{} | cost:{}", LegalEntityRelationV2ServiceImpl.class.getSimpleName(), batch_list.size(), (System.currentTimeMillis() - start));
        return data;
    }
}
